#include <mach/mach.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "common.h"

int main(int argc, const char** const argv) {
  /* Look up the port passed to us by our parent. */
  mach_port_array_t ports;
  mach_msg_type_number_t count;
  kern_return_t kr = mach_ports_lookup(mach_task_self(), &ports, &count);
  if (kr != KERN_SUCCESS) {
    print_mach_error("mach_ports_lookup", kr);
    return 1;
  }

  printf("Child got %d ports\n", count);
  for (int i = 0; i < count; ++i) {
    printf("ports[%d] = %d\n", i, ports[i]);
  }

  /* Debug some common ports. */
  printf("mach_task_self() = %d\n", mach_task_self());
  for (int i = TASK_KERNEL_PORT; i <= TASK_ACCESS_PORT; ++i) {
    mach_port_t p;
    if ((kr = task_get_special_port(mach_task_self(), i, &p)) != KERN_SUCCESS) {
      printf("Special port %d: ", i);
      print_mach_error("task_get_special_port", kr);
      continue;
    }
    printf("task_get_special_port(%d) = %d\n", i, p);
  }

  /* Debug all the ports in our IPC space. */
  mach_port_name_array_t names;
  mach_msg_type_number_t names_count;
  mach_port_type_array_t types;
  mach_msg_type_number_t types_count;
  kr = mach_port_names(mach_task_self(), &names, &names_count, &types, &types_count);
  if (kr != KERN_SUCCESS) {
    print_mach_error("mach_ports_lookup", kr);
    return 1;
  }
  for (int i = 0; i < names_count; ++i) {
    printf("names[%d] = %d %x\n", i, names[i], types[i]);
  }

  /* Say hello to our parent. */
  struct hello_message msg;
  bzero(&msg, sizeof(msg));
  msg.header.msgh_size = sizeof(msg);
  msg.header.msgh_remote_port = ports[0];
  msg.header.msgh_bits = MACH_MSGH_BITS_REMOTE(MACH_MSG_TYPE_COPY_SEND);
  sprintf(msg.message, "I'm a child: %d", getpid());

  if ((kr = mach_msg_send(&msg.header)) == KERN_SUCCESS) {
    printf("Sent message to parent on port %d\n", ports[0]);
  } else {
    print_mach_error("mach_msg_send", kr);
    return 1;
  }

  return 0;
}
