#include <mach/mach.h>
#include <mach/message.h>
#include <stdio.h>
#include <sys/wait.h>
#include <unistd.h>

#include "common.h"
#include "fileport.h"

void wait_for_message(mach_port_t local_port) {
  struct hello_message_request msg;
  bzero(&msg, sizeof(msg));
  msg.req.header.msgh_size = sizeof(msg);
  msg.req.header.msgh_local_port = local_port;

  kern_return_t kr = mach_msg_receive(&msg.req.header);
  if (kr == KERN_SUCCESS) {
    printf("Got request: %s\n", msg.req.message);
  } else {
    printf("mach_msg_receive: %x: %s\n", kr, mach_error_string(kr));
  }
}

int main() {
  kern_return_t kr;

  /* Create listening server port. */
  mach_port_t server_port;
  if ((kr = mach_port_allocate(mach_task_self(), MACH_PORT_RIGHT_RECEIVE, &server_port)) != KERN_SUCCESS) {
    print_mach_error("mach_port_allocate server", kr);
  }

  /* Create a send right for our future child. */
  if ((kr = mach_port_insert_right(mach_task_self(), server_port, server_port, MACH_MSG_TYPE_MAKE_SEND)) != KERN_SUCCESS) {
    print_mach_error("mach_port_insert_right", kr);
  }

  /* Set up the child. */
  mach_port_t ports[] = { server_port };
  if ((kr = mach_ports_register(mach_task_self(), ports, 1)) != KERN_SUCCESS) {
    print_mach_error("mach_ports_register", kr);
  }

  pid_t child = fork();
  if (child < 0) {
    perror("fork");
    return 1;
  } else if (child > 0) {
    /* Parent... */
    printf("Forked %d into %d\n", getpid(), child);

    wait_for_message(server_port);

    /* Reap the child. */
    int stat;
    waitpid(child, &stat, 0);
    printf("Child exited:%d, status:%d\n", WIFEXITED(stat), WEXITSTATUS(stat));
    return 0;
  } else {
    /* Child... */
    printf("Child is %d\n", getpid());
    if (execl("child", "child", NULL)) {
      perror("execl");
      return 1;
    }
  }
}
